@@ -65,16 +65,20 @@ info -w 20012 -o out.265 *file
 info -d // dump
  */
 
+int option_disable_limits = 0;
+
 static struct option long_options[] = {
     //{"write-raw", required_argument, 0, 'w' },
     //{"output",    required_argument, 0, 'o' },
     {(char* const) "dump-boxes", no_argument, 0, 'd'},
+    {(char* const) "disable-limits", no_argument, &option_disable_limits, 1},
     {(char* const) "help",       no_argument, 0, 'h'},
     {(char* const) "version",    no_argument, 0, 'v'},
     {0, 0,                                    0, 0}
 };
 
-const char* fourcc_to_string(uint32_t fourcc)
+// Note: the same function is also exists in common_utils.h, but is not in the public API.
+static const char* fourcc_to_string(uint32_t fourcc)
 {
   static char fcc[5];
   fcc[0] = (char) ((fourcc >> 24) & 0xFF);
@@ -95,6 +99,7 @@ void show_help(const char* argv0)
   //fprintf(stderr,"  -w, --write-raw ID   write raw compressed data of image 'ID'\n");
   //fprintf(stderr,"  -o, --output NAME    output file name for image selected by -w\n");
   fprintf(stderr, "  -d, --dump-boxes     show a low-level dump of all MP4 file boxes\n");
+  fprintf(stderr, "      --disable-limits disable all security limits (do not use in production environment)\n");
   fprintf(stderr, "  -h, --help           show help\n");
   fprintf(stderr, "  -v, --version        show version\n");
 }
@@ -204,6 +209,12 @@ int main(int argc, char** argv)
         heif_free_list_of_compatible_brands(brands);
       }
     }
+    else {
+      if (errno == ENOENT) {
+        std::cerr << "Input file does not exist.\n";
+        exit(10);
+      }
+    }
   }
 
   // ==============================================================================
@@ -215,6 +226,10 @@ int main(int argc, char** argv)
     return 1;
   }
 
+  if (option_disable_limits) {
+    heif_context_set_security_limits(ctx.get(), heif_get_disabled_security_limits());
+  }
+
   struct heif_error err;
   err = heif_context_read_from_file(ctx.get(), input_filename, nullptr);
 
@@ -253,6 +268,15 @@ int main(int argc, char** argv)
 
     printf("image: %dx%d (id=%d)%s\n", width, height, IDs[i], primary ? ", primary" : "");
 
+    heif_image_tiling tiling;
+    err = heif_image_handle_get_image_tiling(handle, true, &tiling);
+    if (err.code) {
+      std::cerr << "Error while trying to get image tiling information: " << err.message << "\n";
+    }
+    else if (tiling.num_columns != 1 || tiling.num_rows != 1) {
+      std::cout << "  tiles: " << tiling.num_columns << "x" << tiling.num_rows
+                << ", tile size: " << tiling.tile_width << "x" << tiling.tile_height << "\n";
+    }
 
     heif_colorspace colorspace;
     heif_chroma chroma;
@@ -273,6 +297,9 @@ int main(int argc, char** argv)
       case heif_colorspace_monochrome:
         printf("monochrome");
         break;
+      case heif_colorspace_nonvisual:
+        printf("non-visual");
+        break;
       default:
         printf("unknown");
         break;
@@ -641,53 +668,60 @@ int main(int argc, char** argv)
       }
     }
 
-    struct heif_image* image;
-    err = heif_decode_image(handle,
-                            &image,
-                            heif_colorspace_undefined,
-                            heif_chroma_undefined,
-                            nullptr);
-    if (err.code) {
-      heif_image_handle_release(handle);
-      std::cerr << "Could not decode image " << err.message << "\n";
-      return 1;
+    // --- camera intrinsic and extrinsic parameters
+
+    if (heif_image_handle_has_camera_intrinsic_matrix(handle)) {
+      heif_camera_intrinsic_matrix matrix{};
+      heif_image_handle_get_camera_intrinsic_matrix(handle, &matrix);
+      printf("  camera intrinsic matrix:\n");
+      printf("    focal length: %f; %f\n", matrix.focal_length_x, matrix.focal_length_y);
+      printf("    principal point: %f; %f\n", matrix.principal_point_x, matrix.principal_point_y);
+      printf("    skew: %f\n", matrix.skew);
     }
 
-    if (image) {
-      uint32_t aspect_h, aspect_v;
-      heif_image_get_pixel_aspect_ratio(image, &aspect_h, &aspect_v);
-      if (aspect_h != aspect_v) {
-        std::cout << "pixel aspect ratio: " << aspect_h << "/" << aspect_v << "\n";
-      }
-
-      if (heif_image_has_content_light_level(image)) {
-        struct heif_content_light_level clli{};
-        heif_image_get_content_light_level(image, &clli);
-        std::cout << "content light level (clli):\n"
-                  << "  max content light level: " << clli.max_content_light_level << "\n"
-                  << "  max pic average light level: " << clli.max_pic_average_light_level << "\n";
-      }
-
-      if (heif_image_has_mastering_display_colour_volume(image)) {
-        struct heif_mastering_display_colour_volume mdcv;
-        heif_image_get_mastering_display_colour_volume(image, &mdcv);
-
-        struct heif_decoded_mastering_display_colour_volume decoded_mdcv;
-        err = heif_mastering_display_colour_volume_decode(&mdcv, &decoded_mdcv);
-
-        std::cout << "mastering display color volume:\n"
-                  << "  display_primaries (x,y): "
-                  << "(" << decoded_mdcv.display_primaries_x[0] << ";" << decoded_mdcv.display_primaries_y[0] << "), "
-                  << "(" << decoded_mdcv.display_primaries_x[1] << ";" << decoded_mdcv.display_primaries_y[1] << "), "
-                  << "(" << decoded_mdcv.display_primaries_x[2] << ";" << decoded_mdcv.display_primaries_y[2] << ")\n";
-
-        std::cout << "  white point (x,y): (" << decoded_mdcv.white_point_x << ";" << decoded_mdcv.white_point_y << ")\n";
-        std::cout << "  max display mastering luminance: " << decoded_mdcv.max_display_mastering_luminance << "\n";
-        std::cout << "  min display mastering luminance: " << decoded_mdcv.min_display_mastering_luminance << "\n";
-      }
+    if (heif_image_handle_has_camera_extrinsic_matrix(handle)) {
+      heif_camera_extrinsic_matrix* matrix;
+      heif_image_handle_get_camera_extrinsic_matrix(handle, &matrix);
+      double rot[9];
+      heif_camera_extrinsic_matrix_get_rotation_matrix(matrix, rot);
+      printf("  camera extrinsic matrix:\n");
+      printf("    rotation matrix:\n");
+      printf("      %6.3f %6.3f %6.3f\n", rot[0], rot[1], rot[2]);
+      printf("      %6.3f %6.3f %6.3f\n", rot[3], rot[4], rot[5]);
+      printf("      %6.3f %6.3f %6.3f\n", rot[6], rot[7], rot[8]);
+      heif_camera_extrinsic_matrix_release(matrix);
     }
 
-    heif_image_release(image);
+
+    uint32_t aspect_h, aspect_v;
+    int has_pasp = heif_image_handle_get_pixel_aspect_ratio(handle, &aspect_h, &aspect_v);
+    if (has_pasp) {
+      std::cout << "pixel aspect ratio: " << aspect_h << "/" << aspect_v << "\n";
+    }
+
+    struct heif_content_light_level clli{};
+    if (heif_image_handle_get_content_light_level(handle, &clli)) {
+      std::cout << "content light level (clli):\n"
+                << "  max content light level: " << clli.max_content_light_level << "\n"
+                << "  max pic average light level: " << clli.max_pic_average_light_level << "\n";
+    }
+
+    struct heif_mastering_display_colour_volume mdcv;
+    if (heif_image_handle_get_mastering_display_colour_volume(handle, &mdcv)) {
+
+      struct heif_decoded_mastering_display_colour_volume decoded_mdcv;
+      err = heif_mastering_display_colour_volume_decode(&mdcv, &decoded_mdcv);
+
+      std::cout << "mastering display color volume:\n"
+                << "  display_primaries (x,y): "
+                << "(" << decoded_mdcv.display_primaries_x[0] << ";" << decoded_mdcv.display_primaries_y[0] << "), "
+                << "(" << decoded_mdcv.display_primaries_x[1] << ";" << decoded_mdcv.display_primaries_y[1] << "), "
+                << "(" << decoded_mdcv.display_primaries_x[2] << ";" << decoded_mdcv.display_primaries_y[2] << ")\n";
+
+      std::cout << "  white point (x,y): (" << decoded_mdcv.white_point_x << ";" << decoded_mdcv.white_point_y << ")\n";
+      std::cout << "  max display mastering luminance: " << decoded_mdcv.max_display_mastering_luminance << "\n";
+      std::cout << "  min display mastering luminance: " << decoded_mdcv.min_display_mastering_luminance << "\n";
+    }
 
     heif_image_handle_release(handle);
   }
