@@ -27,62 +27,83 @@
 
 
 #if HAVE_LIBDE265
-#include "libheif/plugins/decoder_libde265.h"
+#include "plugins/decoder_libde265.h"
 #endif
 
 #if HAVE_X265
-#include "libheif/plugins/encoder_x265.h"
+#include "plugins/encoder_x265.h"
 #endif
 
 #if HAVE_KVAZAAR
-#include "libheif/plugins/encoder_kvazaar.h"
+#include "plugins/encoder_kvazaar.h"
+#endif
+
+#if HAVE_UVG266
+#include "plugins/encoder_uvg266.h"
+#endif
+
+#if HAVE_VVDEC
+#include "plugins/decoder_vvdec.h"
+#endif
+
+#if HAVE_VVENC
+#include "plugins/encoder_vvenc.h"
 #endif
 
 #if HAVE_AOM_ENCODER
-#include "libheif/plugins/encoder_aom.h"
+#include "plugins/encoder_aom.h"
 #endif
 
 #if HAVE_AOM_DECODER
-#include "libheif/plugins/decoder_aom.h"
+#include "plugins/decoder_aom.h"
 #endif
 
 #if HAVE_RAV1E
-#include "libheif/plugins/encoder_rav1e.h"
+#include "plugins/encoder_rav1e.h"
 #endif
 
 #if HAVE_DAV1D
-#include "libheif/plugins/decoder_dav1d.h"
+#include "plugins/decoder_dav1d.h"
 #endif
 
 #if HAVE_SvtEnc
-#include "libheif/plugins/encoder_svt.h"
+#include "plugins/encoder_svt.h"
 #endif
 
-#if HAVE_FFMPEG_HEVC_DECODER
-#include "libheif/plugins/decoder_ffmpeg_hevc.h"
+#if HAVE_FFMPEG_DECODER
+#include "plugins/decoder_ffmpeg.h"
 #endif
 
 #if WITH_UNCOMPRESSED_CODEC
-#include "libheif/plugins/encoder_uncompressed.h"
+#include "plugins/encoder_uncompressed.h"
+#include "plugins/decoder_uncompressed.h"
 #endif
 
 #if HAVE_JPEG_DECODER
-#include "libheif/plugins/decoder_jpeg.h"
+#include "plugins/decoder_jpeg.h"
 #endif
 
 #if HAVE_JPEG_ENCODER
-#include "libheif/plugins/encoder_jpeg.h"
+#include "plugins/encoder_jpeg.h"
+#endif
+
+#if HAVE_OpenH264_DECODER
+#include "plugins/decoder_openh264.h"
 #endif
 
 #if HAVE_OPENJPEG_ENCODER
-#include "libheif/plugins/encoder_openjpeg.h"
+#include "plugins/encoder_openjpeg.h"
 #endif
 
 #if HAVE_OPENJPEG_DECODER
-#include "libheif/plugins/decoder_openjpeg.h"
+#include "plugins/decoder_openjpeg.h"
 #endif
 
-#include "libheif/plugins/encoder_mask.h"
+#include "plugins/encoder_mask.h"
+
+#if HAVE_OPENJPH_ENCODER
+#include "plugins/encoder_openjph.h"
+#endif
 
 std::set<const struct heif_decoder_plugin*> s_decoder_plugins;
 
@@ -131,6 +152,18 @@ void register_default_plugins()
   register_encoder(get_encoder_plugin_kvazaar());
 #endif
 
+#if HAVE_UVG266
+  register_encoder(get_encoder_plugin_uvg266());
+#endif
+
+#if HAVE_VVENC
+  register_encoder(get_encoder_plugin_vvenc());
+#endif
+
+#if HAVE_VVDEC
+  register_decoder(get_decoder_plugin_vvdec());
+#endif
+
 #if HAVE_AOM_ENCODER
   register_encoder(get_encoder_plugin_aom());
 #endif
@@ -151,7 +184,7 @@ void register_default_plugins()
   register_encoder(get_encoder_plugin_svt());
 #endif
 
-#if HAVE_FFMPEG_HEVC_DECODER
+#if HAVE_FFMPEG_DECODER
   register_decoder(get_decoder_plugin_ffmpeg());
 #endif
 
@@ -171,11 +204,20 @@ void register_default_plugins()
   register_decoder(get_decoder_plugin_openjpeg());
 #endif
 
-#if WITH_UNCOMPRESSED_CODEC
-  register_encoder(get_encoder_plugin_uncompressed());
+#if HAVE_OPENJPH_ENCODER
+  register_encoder(get_encoder_plugin_openjph());
 #endif
 
-register_encoder(get_encoder_plugin_mask());
+#if HAVE_OpenH264_DECODER
+  register_decoder(get_decoder_plugin_openh264());
+#endif
+
+#if WITH_UNCOMPRESSED_CODEC
+  register_encoder(get_encoder_plugin_uncompressed());
+  register_decoder(get_decoder_plugin_uncompressed());
+#endif
+
+  register_encoder(get_encoder_plugin_mask());
 }
 
 
