@@ -82,8 +82,7 @@ TEST_CASE("check image handle alpha channel") {
   auto file = GENERATE(FILES, MONO_FILES, ALL_YUV_FILES);
   auto context = get_context_for_test_file(file);
   INFO("file name: " << file);
-  // int expect_alpha = (strchr(file, 'A') == NULL) ? 0 : 1;
-  int expect_alpha = 0; // TODO: fix this
+  int expect_alpha = (strchr(file, 'A') == NULL) ? 0 : 1;
   heif_image_handle *handle = get_primary_image_handle(context);
   int has_alpha = heif_image_handle_has_alpha_channel(handle);
   REQUIRE(has_alpha == expect_alpha);
